/*
** Copyright (C) 2005 CHARY Corentin <iksaif@gmail.com>
**
** Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon
** les termes de la Licence Publique Gnrale GNU publie par la Free Software
**  Foundation
**
** Ce programme est distribu car potentiellement utile, mais SANS AUCUNE 
** GARANTIE, ni explicite ni implicite,  y compris les garanties de 
** commercialisation ou d'adaptation dans un but spcifique.
** Reportez-vous  la Licence Publique Gnrale GNU pour plus de dtails.
**
** Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme
** temps que ce programme
**
** sudoku.c
**
** Changelog
** 
** v0.2 11/12/2005 CHARY Corentin
**  Les grilles de 25x25 avaient tendance  segfaulter ... en effet une grille 
**  de 25x25 a va de 'a'  'y' et pas de 'a'  'z' ...
**
** V0.1 08/12/2005 CHARY Corentin
**  Premire version 
*/


#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include <time.h>


/* 
** Pour l'instant, il faut compiler un binaire par taille de sudoku
** Il y'a deux solutions pour viter a :
** - Metre la taille dans les grilles
** - Ajouter des options qui permettent de choisir la taille en ligne de 
** commande
*/

#ifdef SIZE25x25

#define COLS 25
#define ROWS 25

#define SQR_COLS 5
#define SQR_ROWS 5

#define NB_MIN 1
#define NB_MAX ('y'-'a'+1)

#define ASCII_BASE '`'

#endif /* SIZE25x25 */

#ifdef SIZE9x9

#define COLS 9
#define ROWS 9

#define SQR_COLS 3
#define SQR_ROWS 3

#define NB_MIN 1
#define NB_MAX 9

#define ASCII_BASE '0'

#endif /* SIZE9x9 */

#ifdef SIZE16x16

#define COLS 16
#define ROWS 16

#define SQR_COLS 4
#define SQR_ROWS 4

#define NB_MIN 1
#define NB_MAX ('p'-'a'+1)

#define ASCII_BASE '`'

#endif /* SIZE16x16 */

/* Permet d'avoir diffrent niveaux de verbosite */
#define DEBUG if(verbose == 4) printf("%s:%s:%d ",__FILE__,__FUNCTION__,__LINE__); if(verbose >=2) printf

inline unsigned int getsqr(unsigned int row, unsigned int col) {
	return ((unsigned int) (row/SQR_ROWS))*SQR_COLS+((unsigned int)(col/SQR_COLS));
}

struct sudoku {
	/* Grille */
	unsigned int grid[COLS][ROWS];

	/* 
	** Masques
	** Un 1 represente les nombres possibles 
	** m_case = m_rows&m_cols&m_sqr mais on le stock pour pas refaire tout
	** les calculs quand on a besoin de sa valeur
	** Et parce qu'il est ncessaire pour les choix arbitraires
	*/
	unsigned int m_case[COLS][ROWS];
	unsigned int m_rows[ROWS];
	unsigned int m_cols[COLS];
	unsigned int m_sqr[(COLS/SQR_COLS)*(ROWS/SQR_ROWS)];
	
	/* Choix arbitraires */
	unsigned int arb_x;
	unsigned int arb_y;
	unsigned int arb_nb;

	/* Variables diverses */
	unsigned int unknown;

	/* Nombre de tests */
	double tests;
	
	/* Nombre de tests arbitraires */
	double forks;

	/* Essai prcedent */
	struct sudoku *prev;
};

unsigned int verbose = 0;

/* Init */
struct sudoku * init_grid(struct sudoku *base);

/* Parsage */
unsigned int parse_in(struct sudoku *sudoku);

/* Resolution */
struct sudoku * resolv(struct sudoku *sudoku, struct sudoku *last);

/* Gnration */
void offusc_grid(struct sudoku *sudoku, unsigned int difflvl);

/* Fonctions */
unsigned int basic(struct sudoku *sudoku);

unsigned int single_in_row_col(struct sudoku *sudoku);
unsigned int single_in_box(struct sudoku *sudoku);
unsigned int pointing_pairs(struct sudoku *sudoku);

struct sudoku * random_choice(struct sudoku *sudoku);
void cancel_random_choice(struct sudoku *sudoku, struct sudoku **last);

/* Verification */
unsigned int check_grid(struct sudoku *sudoku);

/* Maj d'une case */
void set_case(struct sudoku *sudoku, unsigned int x, unsigned int y, unsigned int nb, unsigned int update_other_case);

/* Divers */
unsigned int get_first_one_pos(unsigned int mask);
unsigned int count_ones(unsigned int mask);

/* Affichage */
void show_grid(struct sudoku *sudoku);
void show_nice_grid(struct sudoku *sudoku);
void show_mask(unsigned int mask);
void show_infos(struct sudoku *sudoku);

/* Free ...*/
void free_sudoku(struct sudoku *sudoku);

int main(int argc, char ** argv)
{
	struct sudoku *base=NULL,*last;
	unsigned int c,difflvl=0;
	extern char *optarg;
	extern int optind;
	extern int opterr;
	extern int optopt;

	unsigned int sflg=0,mflg=0,nflg=0;
	int errflg = 0;
	opterr=0;
          
	/* Commandes ... */
	while ((c = getopt(argc, argv, "hnsm:v:")) != EOF)
	{
		
		switch (c) {

		case 's':
			if (mflg)
				errflg++;
			else
				sflg++;
			break;
		case 'm':
			if (sflg)
				errflg++;
			else
				if(atoi(optarg) < 1 || atoi(optarg) > 5)
					errflg++;
				else {
					mflg++;
					difflvl=atoi(optarg);
				}
			break;

		case 'v':
			if(atoi(optarg) < 1 || atoi(optarg) > 4)
				errflg++;
			else
				verbose=atoi(optarg);
			break;
		case 'n':
			nflg=1;
			break;
		case 'h':
			errflg++;
			break;
		case '?':
			printf("Option inconnue:  %c\n",optopt);
			errflg++;
		}
	}
	if (errflg) {
		fprintf(stderr,	
			"Usage: %s [-n] [-s|-m diff_lvl] [-v verbose_lvl]\n"
			"\t-n\t\tAfficher la grille avec les dcorations\n"
			"\t-s\t\tRsoudre (Par defaut)\n"
			"\t-c diff_lvl\tCrer une grille de difficulte diff_lvl {1,..,5}\n"
			"\n"
			"\t-v verbose_lvl\tNiveau de Verbose: {1,2,3,4}\n"
			,argv[0]);
		return EXIT_FAILURE;
	}

	/* Initialisation */
	base=init_grid(base);

	/* Le haut de la "pile" est donc la grille de base */
	last=base;

	/* Remplissage de la grille */
	if(parse_in(base)) return EXIT_FAILURE;

	if(verbose>=2) show_nice_grid(base);

	/* On verifie que la grille qu'on nous donne est ok */
	if(check_grid(base)) {
		fprintf(stderr,"Grille Impossible\n");
		return EXIT_FAILURE;
	}

	last=resolv(base,last);

	if(mflg) /* Si on gnre, on rajoute des vides */
		offusc_grid(last,difflvl);

	if(nflg) /* Suivant l'option */
		show_nice_grid(last);
	else
		show_grid(last);

	free_sudoku(last);
	return EXIT_SUCCESS;
}

/*
** Initialise la grille, et renvoit un pointeur vers cette grille
*/
struct sudoku * init_grid(struct sudoku *base)
{
        unsigned int i,j,c;
	unsigned int nb_max_bin=0; /* Pour memset, on utilise un octet remplit de 1 */

        for(c=1;c<=NB_MAX;++c) {
                nb_max_bin|=(1<<c);
        }

        /* Init */
        base = malloc(sizeof(struct sudoku));
	
        for(j=0;j<ROWS;++j) {
                for(i=0;i<COLS;++i) {
                        base->grid[i][j] = 0;
			base->m_case[i][j] = nb_max_bin;
                }
	}

        /* Masques, par defaut, tout est possible */
        for(i=0;i<COLS;++i) {
                base->m_cols[i]=nb_max_bin;
	}
        for(i=0;i<ROWS;++i) {
                base->m_rows[i]=nb_max_bin;
        }
        for(i=0;i<(COLS/SQR_COLS)*(ROWS/SQR_ROWS);++i) {
                base->m_sqr[i]=nb_max_bin;
	}
	
        /* Aucun choix arbitraire au dbut */
        base->arb_x=COLS;
        base->arb_y=ROWS;
        base->arb_nb=0;

        /* .. */
        base->unknown=COLS*ROWS;

        /* Stats */
        base->tests=0;
        base->forks=0;

        /* Rglage des pointeurs */
        base->prev = NULL;

	return base;
}

/*
** Parse l'entre std pour en faire une grille
*/
unsigned int parse_in(struct sudoku *sudoku)
{
	unsigned int i=0,j=0,n=0,c,nb_max_bin=0;
	
	for(c=1;c<=NB_MAX;++c) {
		nb_max_bin|=(1<<c);
	}

	
	while((c = getchar()) != EOF && n < COLS*ROWS) {
		if(c >= ' ') { /* On ne traite que les char imprimables, cf 'man ascii' */
			/* Calcul en fonction du numero */
			i=(n%COLS);
			j=(n-i)/ROWS;

			/* Tests .. */
			if(c==' ' || c==ASCII_BASE) { /* c = ' ' ou '0' => c = ??? */
				c=0;
			}
			else if(c >= (NB_MIN+ASCII_BASE) && c <= (NB_MAX+ASCII_BASE)) c-=ASCII_BASE;
			else { /* o_O */
				fprintf(stderr,"%c n'a strictement rien a faire la ...\n",c);
				return 1;
			}

			sudoku->grid[i][j]=c;

			/* Nombre connu = Maj des masques */
			if(c!=0) {
				sudoku->m_case[i][j]=(1<<c);
				set_case(sudoku,i,j,c,0);
				sudoku->unknown--;
			} else {
				sudoku->m_case[i][j]=nb_max_bin;
			}
			
			n++;
		}
	}

	return 0;
}

struct sudoku * resolv(struct sudoku *sudoku, struct sudoku *last)
{
	unsigned int lunknown=COLS*ROWS;
	unsigned int stage;

	last=sudoku;

	while(last->unknown != 0) {
		if(verbose>=2) show_nice_grid(last);
		if(verbose>=1) show_infos(last);

		lunknown = last->unknown;
		stage=0;

		if(stage == 0) if(!basic(last)) stage++;
		if(stage == 1) if(!single_in_row_col(last)) stage++;
		if(stage == 2) if(!single_in_box(last)) stage++;
		if(stage == 3) if(!pointing_pairs(last)) stage++;
		if(stage == 4) { 
			if(check_grid(last)) { /* Si la grille est impossible, on annule */
				cancel_random_choice(last,&last);
				continue; /* on refait les tests de base, a a des chances de marcher */
			}
			last->forks++;
			last = random_choice(last);
		}

		last->tests+=(stage+1);
		
		/*
		** La rsolution bte et mchante ne fonctionne pas
		** On applique donc des trucs plus jolis
		** Et si rien ne marche, on essaye un truc au hasard ...
		*/

	}
	
	return last;
}

/* Gnration */
void offusc_grid(struct sudoku *sudoku, unsigned int difflvl)
{
        unsigned int i,j,c,nb_max_bin=0;
	
	srand(time(NULL));

        for(c=1;c<=NB_MAX;++c) {
                nb_max_bin|=(1<<c);
        }

        for(j=0;j<ROWS;++j) {
                for(i=0;i<COLS;++i) {
			if(rand()%6 < difflvl) {
				sudoku->m_case[i][j]=nb_max_bin;
				set_case(sudoku,i,j,0,1);
			}
		}
	}

}

/* Test basique */
unsigned int basic(struct sudoku *sudoku)
{
	unsigned int lunknown = sudoku->unknown;
	unsigned int i,j,c;
	
	DEBUG("Test Basic\n");
	
	for(j=0;j<ROWS;++j) {
		for(i=0;i<COLS;++i) {
			if(sudoku->grid[i][j] == 0) {
			    sudoku->m_case[i][j]&=sudoku->m_rows[j] & sudoku->m_cols[i] & sudoku->m_sqr[getsqr(i,j)];
				if(count_ones(sudoku->m_case[i][j]) == 1) {
					
					c=get_first_one_pos(sudoku->m_case[i][j]);
					
					set_case(sudoku,i,j,c,0);
					sudoku->unknown--;
				}
			}
		}
	}
	
	return (sudoku->unknown != lunknown);
}

/*
** Si un chiffre est une seule fois possible dans une ligne ou une colonne
** Alors, c'est qu'il est l ou il est une seule fois possible
*/
unsigned int single_in_row_col(struct sudoku *sudoku)
{
        unsigned int lunknown = sudoku->unknown;
        unsigned int i,j,c,n,k;

	DEBUG("Test single_in_row_col \n");

        for(j=0;j<ROWS;++j) {
		for(i=0;i<COLS;++i) {
			if(!sudoku->grid[i][j]) {
				for(n=1;n<=NB_MAX && !sudoku->grid[i][j];n++) if( sudoku->m_case[i][j] & (1 << n) ) {
					/* Pour tout les nombres, si ils sont possibles */
					
					c=0;
					for( k=0;k<ROWS;k++)  /* La ligne */
						if( !sudoku->grid[k][j] && (sudoku->m_case[k][j] & (1 << n)) ) c++;
					
					if(c==1) /* Un seul */
					{
						sudoku->unknown--;
						sudoku->m_case[i][j] = (1 << n);
						set_case(sudoku,i,j,n,1);
					}
					
					c=0;
					for( k=0;k<COLS;k++)  /* La colonne */
						if( !sudoku->grid[i][k] && (sudoku->m_case[i][k] & (1 << n)) ) c++;
					
					if(c==1) /* Un Seul */
					{
						sudoku->unknown--;
						sudoku->m_case[i][j] = (1 << n);
						set_case(sudoku,i,j,n,1);
					}
				}
			}
		}
	}

	return (sudoku->unknown != lunknown);
}

/*
** Pareil, pour un carr
*/
unsigned int single_in_box(struct sudoku *sudoku)
{
	unsigned int i,j,k,c,l,n;
	unsigned int nb_max_bin=0;
	unsigned int mask,present;
	unsigned int posx[NB_MAX-NB_MIN],posy[NB_MAX-NB_MIN];
	unsigned int lunknown = sudoku->unknown;
	
	DEBUG("Test single_in_box \n");

        for(c=1;c<=NB_MAX;++c) {
                nb_max_bin|=(1<<c);
        }
	
	/* On verifie maintenant tout les carrs */
	for(k=0;k<ROWS;k+=SQR_ROWS) {
		for(l=0;l<COLS;l+=SQR_COLS) {
			present=0;
			mask=0;
			memset(&posx,0,sizeof(posx));
			memset(&posy,0,sizeof(posy));
			for(j=k;j<SQR_ROWS+k;j++) {
				for(i=l;i<SQR_COLS+l;i++) {
					if(!sudoku->grid[i][j]) {
						for(n=1;n<=NB_MAX;n++) {
							c=(1<<n);
							if(sudoku->m_case[i][j]&c) { 
								if(present&c) { /* Si il est prsent et encore dans le masque */
									if(mask&c) {
										mask^=c; /* On le vire du masque final */
									}
								} else {
									present|=c;
									mask|=c; /* Sinon on l'ajoute */
									posx[n]=i;
									posy[n]=j;
								}
							}
						}
					}
				}
			}
			
			/* Fin de ce carr, on fait les modifs si y'en a  faire ... */
			for(n=1;n<=NB_MAX;n++) {
				c=(1<<n);
				if(mask&c && !sudoku->grid[posx[n]][posy[n]]) { /* Le nombre n est prsent une seule fois */
					
					sudoku->unknown--;
					sudoku->m_case[posx[n]][posy[n]] = c;
					set_case(sudoku,posx[n],posy[n],n,1);
				}
			}
		}
	}
	
        return (sudoku->unknown != lunknown);
}


/*
** Voir : http://www.scanraid.com/BasicStrategies.htm#PP
** Cette fonction ne trouve pas de solution mais limine des possibilites
*/
unsigned int pointing_pairs(struct sudoku *sudoku)
{
        unsigned int i,j,k,l,n,c;
	unsigned int posx[COLS];
	unsigned int posy[ROWS];
	unsigned int modif=0;

	DEBUG("Test Pointing Pairs\n");

	for(k=0;k<ROWS;k+=SQR_ROWS) {
                for(l=0;l<COLS;l+=SQR_COLS) {
			for(n=1;n<=NB_MAX;n++) { /* Tout les nombres */
				c=0;
				for(j=k;j<SQR_ROWS+k && c<=4;j++) { /* A partir de 4, a sert plus  rien */
					for(i=l;i<SQR_COLS+l && c<=4;i++) {
						if(sudoku->grid[i][j]==0 && sudoku->m_case[i][j]&(1<<n)) {
							posx[c]=i; /* On stocke */
							posy[c]=j;
							c++;	
						} else if(sudoku->grid[i][j]==n) {
							c=NB_MAX; /* Ce nombre est connus, on passe */
						}
					}
				}

				/* TODO: Refaire et gnraliser .... */
				if(c==2) {
					if(posx[0]==posx[1]) { /* sur une mme colonne */
						for(j=0;j<ROWS;j++) {
							/* Si : case diffrente non connue, et nombre possible dans cette case */
							if(sudoku->grid[posx[0]][j]==0 && j!=posy[0] 
							   && j!=posy[1] && (sudoku->m_case[posx[0]][j]&(1<<n))) { 
								DEBUG("%d est pas possible en %dx%d\n",n,posx[0],j);
								sudoku->m_case[posx[0]][j]^=(1<<n);
								modif++;
							}
						}
					}

                                        if(posy[0]==posy[1]) { /* sur une mme ligne */
                                                for(i=0;i<ROWS;i++) {
                                                        /* Si : case diffrente non connue, et nombre possible dans cette case */
                                                        if(sudoku->grid[i][posy[0]]==0 && i!=posx[0]
                                                           && i!=posx[1] && (sudoku->m_case[i][posy[0]]&(1<<n))) {
								DEBUG("%d est pas possible en %dx%d\n",n,i,posy[0]);
								sudoku->m_case[i][posy[0]]^=(1<<n);
								modif++;
                                                        }
                                                }
                                        }
				}

                                if(c==3) { /* triples */
                                        if(posx[0]==posx[1] && posx[0]==posx[2]) { /* sur une mme colonne */
                                                for(j=0;j<ROWS;j++) {
                                                        /* Si : case diffrente non connue, et nombre possible dans cette case */
                                                        if(sudoku->grid[posx[0]][j]==0 && j!=posy[0] && j!=posy[2]
                                                           && j!=posy[1] && (sudoku->m_case[posx[0]][j]&(1<<n))) {
                                                                DEBUG("%d est pas possible en %dx%d\n",n,posx[0],j);
                                                                sudoku->m_case[posx[0]][j]^=(1<<n);
                                                                modif++;
                                                        }
                                                }
                                        }
					
                                        if(posy[0]==posy[1] && posy[0]==posy[2]) { /* sur une mme ligne */
                                                for(i=0;i<ROWS;i++) {
                                                        /* Si : case diffrente non connue, et nombre possible dans cette case */
                                                        if(sudoku->grid[i][posy[0]]==0 && i!=posx[0] && i!=posx[2]
                                                           && i!=posx[1] && (sudoku->m_case[i][posy[0]]&(1<<n))) {
                                                                DEBUG("%d est pas possible en %dx%d\n",n,i,posy[0]);
                                                                sudoku->m_case[i][posy[0]]^=(1<<n);
                                                                modif++;
                                                        }
                                                }
                                        }
                                }
			}
			
		}
	}

	return modif;
}


/*
** Fait un choix arbitraire
*/
struct sudoku * random_choice(struct sudoku *sudoku)
{
	unsigned i,j,c,min=NB_MAX,minx=0,miny=0;

	/* Sauvegarde */
	struct sudoku *new;
	new = malloc(sizeof(struct sudoku));
	*new=*sudoku;

	/* Choix arbitraire */
	for(j=0;j<ROWS && min>2;++j) {
                for(i=0;i<COLS && min>2;++i) {
			if(!new->grid[i][j]) {
				c=count_ones(new->m_case[i][j]); 
				if(c >= 2 && c < min) { /* On cherche la case avec le moins de possibilites */
					min=c;
					minx=i;
					miny=j;
				}
			}
		}
	}
	
	c=get_first_one_pos(new->m_case[minx][miny]);

	DEBUG("On choisis arbitrairement de metre %dx%d  %d\n",minx,miny,c);
	
	new->m_case[minx][miny] = (1 << c);
	set_case(new,minx,miny,c,1);
	
	new->arb_x=minx;
	new->arb_y=miny;
	new->arb_nb=c;
	
	new->unknown--;
	
	new->prev=sudoku;
	
	return new;
}

/*
** Annule le dernier choix arbitraire, et le met dfinitivement comme impossible
*/
void cancel_random_choice(struct sudoku *sudoku, struct sudoku **last)
{
	DEBUG("Annulation du dernier changement \n");

	*last=sudoku->prev;
      
	/* Ancien nombre mis comme impossible */
	(*last)->m_case[sudoku->arb_x][sudoku->arb_y]^=(1<<sudoku->arb_nb);

	/* Rcuperation des stats */
	(*last)->tests+=sudoku->tests;
	(*last)->forks+=sudoku->forks;
	
	free(sudoku);
}

/*
** Verifie qu'une grille est ok
** 0 = ok
** 1 = pas bon
*/
unsigned int check_grid(struct sudoku *sudoku)
{
        unsigned int i,j,k,l,c,mask;

	/* Si un masque est totalement a 0, c'est que y'a une couille */
        for(j=0;j<ROWS;++j) {
                for(i=0;i<COLS;++i) {
			if(sudoku->grid[i][j]==0) {
				if(sudoku->m_case[i][j]==0) {
					DEBUG("Masque %dx%d cass\n",i,j);
					return 1; /* pas bon */
				}
			}
                }
        }

	/* Pour chaque ligne, on verifie que y'a pas un nombre en double */
	for(j=0;j<ROWS;++j) {
		mask=0;
		for(i=0;i<COLS;++i) {
			if(sudoku->grid[i][j]) {
				c=(1 << sudoku->grid[i][j]);
				if(mask&c) { /* Le nombre est dj prsent */
					DEBUG("Deux %d sur la ligne %d\n",sudoku->grid[i][j],j);
					return 1; /* pas bon */
				}
				mask|=c; /* on l'ajoute */
			}
		}
	}
        
        /* Pour chaque colonne, on verifie que y'a pas un nombre en double */
        for(i=0;i<COLS;++i) {
                mask=0;
                for(j=0;j<ROWS;++j) {
			if(sudoku->grid[i][j]) {
				c=(1 << sudoku->grid[i][j]);
				if(mask&c) { /* Le nombre est dj prsent */
					DEBUG("Deux %d sur la colonne %d\n",sudoku->grid[i][j],i);
					return 1; /* pas bon */
				}
				mask|=c; /* on l'ajoute */
			}
                }
        }

	/* On verifie maintenant tout les carrs */
	for(k=0;k<ROWS;k+=SQR_ROWS) {
		for(l=0;l<COLS;l+=SQR_COLS) {
			mask=0;
			for(j=k;j<SQR_ROWS+k;j++) {
				for(i=l;i<SQR_COLS+l;i++) {
					if(sudoku->grid[i][j]) { 
						c=(1 << sudoku->grid[i][j]);
						if(mask&c) { /* Le nombre est dj prsent */
							DEBUG("Deux %d dans le carr %dx%d\n",sudoku->grid[i][j],k,l);
							return 1; /* pas bon */
						}
						mask|=c; /* on l'ajoute */
					}
				}
			}
		}
	}
	return 0;
}


/* Met une case a n */
void set_case(struct sudoku *sudoku, unsigned int x, unsigned int y, unsigned int nb, unsigned int update_other_case)
{
	unsigned int i,j,sqr=getsqr(x,y);
	
	DEBUG("Passage de la case %dx%d  %d\n",x,y,nb);

	sudoku->grid[x][y]=nb;
	
	sudoku->m_rows[y]^=(1<<nb);
	sudoku->m_cols[x]^=(1<<nb);
	sudoku->m_sqr[sqr]^=(1<<nb);
	
	if(update_other_case) {
		for(j=0;j<ROWS;++j) {
			for(i=0;i<COLS;++i) {
				if(i==x || j==y || getsqr(i,j) == sqr)
					if(sudoku->m_case[i][j] & (1 << nb))
						sudoku->m_case[i][j]^= (1 << nb);
			}
		}
	}
}

/* Trouve la position du premier 1 */
unsigned int get_first_one_pos(unsigned int mask)
{
        unsigned int i;
        for(i=1;i<=NB_MAX;i++) {
                if(mask&(1<<i)) return i;
	}
        return 0;
}

/* Compte le nombre de 1e chiffre en binaire */
unsigned int count_ones(unsigned int mask)
{
	unsigned int i,r=0;
	for(i=1;i<=NB_MAX;i++)
		r+=((mask&(1<<i))?1:0);

	return r;
}

/* Affiche grille moche, mais reutilisable par le prog lui meme */
void show_grid(struct sudoku *sudoku)
{
	unsigned int i,j,c;

	for(j=0;j<ROWS;j++) {
		for(i=0;i<COLS;i++) {
			c=sudoku->grid[i][j];
			c=(c==0) ? ' ': c+ASCII_BASE;
			printf("%c",c);
		}
		printf("\n");
	}
}

/* Affiche une jolie grille */
void show_nice_grid(struct sudoku *sudoku)
{
	unsigned int i,j,c;
	
	for(i=0;i<COLS;i++)
		printf("+--");
	printf("+");
	printf("\n");
	
	for(j=0;j<ROWS;j++) {
		printf("|");
		for(i=0;i<COLS;i++) {
			c=sudoku->grid[i][j];
			c=(c==0) ? ' ': c+ASCII_BASE;
			printf(" %c",c);
			if(i%SQR_COLS==SQR_COLS-1)
				printf("|");
			else
				printf(" ");
		}
		if(j%SQR_ROWS==SQR_ROWS-1) {
			printf("\n");
			for(i=0;i<COLS;i++)
				printf("+--");
			printf("+");
		}
		printf("\n");
	}
	
	if(sudoku->unknown)
		printf("Inconnues = %d\n",sudoku->unknown);
	
	printf("Tests %1.0lf\n"
	       "Forks %1.0lf\n",
	       sudoku->tests,
	       sudoku->forks);
}

/* Infos sur l'tat d'avancement */
void show_infos(struct sudoku *sudoku) 
{

	printf("Etat: %d/%d, Tests: %1.0lf, Fork %1.0lf\n",
	       sudoku->unknown,
	       (COLS*ROWS),
	       sudoku->tests,
	       sudoku->forks
	       );
	
}

/* Affiche un masque binaire */
void show_mask(unsigned int mask)
{
	unsigned int i;
	
	for(i=0;i<=NB_MAX;++i)
		printf("%d",(mask&(1<<i))?1:0);
	printf("\n");
	
}

/* Libre la mmoire */
void free_sudoku(struct sudoku *sudoku)
{
	struct sudoku *tmp;

	while(sudoku) {
		tmp = sudoku->prev;
		free(sudoku);
		sudoku = tmp;
	}
}
